/*  Couch Potato's Planner- written by Brian Hill, perelandra@iname.com
    
    This program was written for BeOS (www.be.com) and provides an interface
    for searching the internet for TV schedules of your favorite programs.
    
    
    --Protected under General Public License--
    
    Copyright (C) 2001,  Brian Hill <perelandra@iname.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "ReloadDeskbarIcons.h"

const char* app_sig = "application/x-vnd.CT-ReloadDeskbarIcons";
const char* RDIsettingsDir = "/boot/home/config/settings/ReloadDeskbarIcons";
const rgb_color bg_color = {216,216,216,0};
const rgb_color enabled_color = {200,0,0,0};
const rgb_color disabled_color = {0,0,0,0};
const rgb_color selected_color = {178,178,178,255};

int main(int argc, char** argv)
{	bool reload = false;
	if(argc>1)
	{	if(strcmp(argv[1],"-r")==0)
		{	reload = true; }
		else
		{	printf("ReloadDeskbarIcons usage:\n"
					"ReloadDeskbaricons [-r]\n\n"
					"ReloadDeskbarIcons& will simply run the application\n"
					"If the -r option is used, no GUI will appear and "
					"all enabled BMessages from the previous session will "
					"be sent.\n");
			return(0);
		}
	}
	RDIApp myApp(reload);
	myApp.Run();
	return(0);
}
RDIApp::RDIApp(bool rel) :BApplication(app_sig), reload(rel)
{	aWindow = new RDIWindow(BRect(200,50,500,250)); }
RDIApp::~RDIApp()
{	}
void RDIApp::ReadyToRun()
{	if(!reload) aWindow->Show();
	else
	{	BMessage send_msg(SEND_ALL_MSG);
		aWindow->MessageReceived(&send_msg);
		be_app->PostMessage(B_QUIT_REQUESTED);
	}
	return;
}
RDIWindow::RDIWindow(BRect size)
	:BWindow(size, "ReloadDeskbarIcons Preferences", B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE)
	,selectedMsg(NULL), selectedMType(0)
{	Lock();
	BRect viewRect(Bounds());
	mainView = new BView(viewRect, "Main Prefs View", B_FOLLOW_ALL, B_WILL_DRAW);
	mainView->SetViewColor(bg_color);
	//Apps List
	font_height FontAttributes;
	be_plain_font->GetHeight(&FontAttributes);
	float FontHeight = ceil(FontAttributes.ascent) + ceil(FontAttributes.descent);
	viewRect = Bounds();
	viewRect.InsetBy(BORDER_SIZE + 2, BORDER_SIZE + 2);
	viewRect.bottom = viewRect.top + 10*FontHeight + 25;//Change this
	//viewRect.right -= B_V_SCROLL_BAR_WIDTH;
	viewRect.bottom -= B_H_SCROLL_BAR_HEIGHT;
	msgLView = new BListView(viewRect, "App List View", B_SINGLE_SELECTION_LIST, B_FOLLOW_ALL_SIDES, B_WILL_DRAW);
	msgLView->SetSelectionMessage(new BMessage(MSG_SELECTION_CHANGED));
	msgSView = new BetterScrollView("Apps List Scroll View", msgLView, B_FOLLOW_NONE);
	mainView->AddChild(msgSView);
	//Buttons
	viewRect.OffsetTo(msgSView->Frame().right + 3, msgSView->Frame().top);
	viewRect.bottom = viewRect.top + 10;
	viewRect.right = viewRect.left + be_plain_font->StringWidth("Add Message") + 14;
	msgUpB = new BButton(viewRect, "Up Button", "Move Up", new BMessage(MOVE_MSG_UP), B_FOLLOW_NONE, B_WILL_DRAW);
	mainView->AddChild(msgUpB);
	viewRect.OffsetTo(viewRect.left, msgUpB->Frame().bottom + 3);
	msgDownB = new BButton(viewRect, "Down Button", "Move Down", new BMessage(MOVE_MSG_DOWN), B_FOLLOW_NONE, B_WILL_DRAW);
	mainView->AddChild(msgDownB);
	viewRect.OffsetTo(viewRect.left, msgDownB->Frame().bottom + 3);
	msgAddB = new BButton(viewRect, "Add Button", "Add Message", new BMessage(ADD_MSG), B_FOLLOW_NONE, B_WILL_DRAW);
	mainView->AddChild(msgAddB);
	//Included items menu
	viewRect.OffsetTo(msgUpB->Frame().left, msgAddB->Frame().bottom + 3);
	viewRect.right += mainView->Bounds().right;
	includedItemsMenu = new BPopUpMenu("Add Included", false, false);
	includedItemsMenu->AddItem(new BMenuItem("Add All Apps", new BMessage(ADD_INC_ALL)));
	includedItemsMenu->AddItem(new BMenuItem("BeAIM", new BMessage(ADD_INC_BEAIM)));
	includedItemsMenu->AddItem(new BMenuItem("GimICQ", new BMessage(ADD_INC_GIMICQ)));
	includedItemsMenu->AddItem(new BMenuItem("ICBM", new BMessage(ADD_INC_ICBM)));
	itemField = new BMenuField(viewRect, "Items", NULL, includedItemsMenu);
	mainView->AddChild(itemField);
	viewRect = msgAddB->Frame();
	viewRect.OffsetTo(viewRect.left, msgAddB->Frame().bottom + msgAddB->Frame().Height() + 6);
	msgRemB = new BButton(viewRect, "Remove Button", "Remove Msg", new BMessage(REMOVE_MSG),  B_FOLLOW_NONE, B_WILL_DRAW);
	mainView->AddChild(msgRemB);
	viewRect.OffsetTo(viewRect.left, msgRemB->Frame().bottom + 3);
	msgSendB = new BButton(viewRect, "Send Message", "Send All Now", new BMessage(SEND_ALL_MSG), B_FOLLOW_NONE, B_WILL_DRAW);
	mainView->AddChild(msgSendB);
	//appsSView->ResizeTo(appsSView->Bounds().Width(), msgSendB->Frame().bottom - appsSView->Frame().top);
	//settings box
	viewRect.OffsetTo(BORDER_SIZE, msgSView->Frame().bottom + 3);
	viewRect.right = msgUpB->Frame().right;
	viewRect.bottom = viewRect.top + 100;
	msgBox = new BBox(viewRect, "BMessage Settings", B_FOLLOW_NONE, B_WILL_DRAW);
	msgBox->SetLabel("BMessage Settings");
	//App settings
	viewRect.OffsetTo(10,15);
	viewRect.bottom = viewRect.top + 10;
	msgEnabledRB = new BRadioButton(viewRect, "Enabled CB", "Enabled", new BMessage(TOGGLE_ENABLED), B_FOLLOW_NONE, B_WILL_DRAW);
	msgEnabledRB->ResizeToPreferred();
	msgBox->AddChild(msgEnabledRB);
	viewRect.OffsetTo(msgEnabledRB->Frame().right + 10, msgEnabledRB->Frame().top);
	msgDisabledRB = new BRadioButton(viewRect, "Disabled RB", "Disabled", new BMessage(TOGGLE_ENABLED), B_FOLLOW_NONE, B_WILL_DRAW);
	msgDisabledRB->ResizeToPreferred();
	msgBox->AddChild(msgDisabledRB);
	viewRect.OffsetTo(msgBox->Bounds().right - msgEnabledRB->Frame().left - be_plain_font->StringWidth("Message Command: WWWW") - 15, viewRect.top - 2);
	viewRect.right = msgBox->Bounds().right - msgEnabledRB->Frame().left;
	msgCmdTC = new BTextControl(viewRect, "Message Command", "Message Command:", NULL, B_FOLLOW_NONE, B_WILL_DRAW);
	msgCmdTC->SetDivider(be_plain_font->StringWidth(msgCmdTC->Label()) + 4);
	msgCmdTC->TextView()->SetMaxBytes(4);
	msgBox->AddChild(msgCmdTC);
	viewRect.OffsetTo(msgEnabledRB->Frame().left, msgEnabledRB->Frame().bottom + 3);
	viewRect.right = msgBox->Bounds().right - msgEnabledRB->Frame().left;
	viewRect.bottom = viewRect.top + 5;
	msgNameTC = new BTextControl(viewRect, "Message Name", "Message Name:", NULL, B_FOLLOW_NONE, B_WILL_DRAW);
	msgNameTC->SetDivider(be_plain_font->StringWidth(msgNameTC->Label()) + 4);
	msgBox->AddChild(msgNameTC);
	viewRect.OffsetTo(msgEnabledRB->Frame().left, msgNameTC->Frame().bottom + 5);
	viewRect.right = msgBox->Bounds().right - B_V_SCROLL_BAR_WIDTH - msgEnabledRB->Frame().left;
	viewRect.bottom = viewRect.top + 4*FontHeight + 10;//Change this
	dataCLView = new ColumnListView(viewRect, &dataCView, NULL, B_FOLLOW_NONE, B_WILL_DRAW|B_FRAME_EVENTS|B_NAVIGABLE, B_SINGLE_SELECTION_LIST, false, true, true, true, B_FANCY_BORDER);
	dataCLView->AddColumn(new CLVColumn("Type",50.0,CLV_SORT_KEYABLE, be_plain_font->StringWidth("Type")+15));
	dataCLView->AddColumn(new CLVColumn("Name",160.0,CLV_SORT_KEYABLE, be_plain_font->StringWidth("Name")+15));
	dataCLView->AddColumn(new CLVColumn("Value",150.0,CLV_SORT_KEYABLE, be_plain_font->StringWidth("Value")+15));
	dataCLView->SetSelectionMessage(new BMessage(DATA_SELECTION_CHANGED));
	msgBox->AddChild(dataCView);
	viewRect.OffsetTo(msgEnabledRB->Frame().left - 2, dataCView->Frame().bottom + 3);
	viewRect.right = viewRect.left + (dataCView->Frame().Width() - 3)/2.0;
	viewRect.bottom = viewRect.top + FontHeight;
	dataNewB = new BButton(viewRect, "New BMessage Data", "New Message Data Item", new BMessage(NEW_DATA_ITEM), B_FOLLOW_NONE, B_WILL_DRAW);
	msgBox->AddChild(dataNewB);
	viewRect.OffsetTo(dataNewB->Frame().right + 3, viewRect.top);
	dataRemB = new BButton(viewRect, "Remove BMessage Data", "Remove Message Data Item", new BMessage(REMOVE_DATA_ITEM), B_FOLLOW_NONE, B_WILL_DRAW);
	msgBox->AddChild(dataRemB);
	
	viewRect.OffsetTo(msgEnabledRB->Frame().left, dataNewB->Frame().bottom + 3);
	viewRect.right = msgBox->Bounds().right/2.0;
	viewRect.bottom = viewRect.top + FontHeight;
	dataTypeMenu = new BPopUpMenu("Data Type");
	dataTypeMenu->AddItem(new BMenuItem("Bool", NULL));
	dataTypeMenu->AddItem(new BMenuItem("Int8", NULL));
	dataTypeMenu->AddItem(new BMenuItem("Int16", NULL));
	dataTypeMenu->AddItem(new BMenuItem("Int32", NULL));
	dataTypeMenu->AddItem(new BMenuItem("Int64", NULL));
	dataTypeMenu->AddItem(new BMenuItem("Float", NULL));
	dataTypeMenu->AddItem(new BMenuItem("Double", NULL));
	dataTypeMenu->AddItem(new BMenuItem("String", NULL));
	//dataTypeMenu->AddItem(new BMenuItem("Point", NULL));
	//dataTypeMenu->AddItem(new BMenuItem("Rect", NULL));
	//dataTypeMenu->AddItem(new BMenuItem("Ref", NULL));
	//dataTypeMenu->AddItem(new BMenuItem("Message", NULL));
	//dataTypeMenu->AddItem(new BMenuItem("Messenger", NULL));
	//dataTypeMenu->AddItem(new BMenuItem("Pointer", NULL));
	//dataTypeMenu->AddItem(new BMenuItem("Flat", NULL));
	dataTypeMenu->ItemAt(0)->SetMarked(true);
	dataTypeField = new BMenuField(viewRect, "Data Field:", "Data Type:", dataTypeMenu);
	dataTypeField->SetDivider(be_plain_font->StringWidth(dataTypeField->Label()) + 4);
	msgBox->AddChild(dataTypeField);
	viewRect.OffsetTo(dataTypeField->Frame().right + 3, viewRect.top + 1);
	viewRect.right = msgBox->Bounds().right - msgEnabledRB->Frame().left;
	dataValTC = new BTextControl(viewRect, "Data Value", "Data Value:", NULL, B_FOLLOW_NONE, B_WILL_DRAW);
	dataValTC->SetDivider(be_plain_font->StringWidth(dataValTC->Label()) + 4);
	msgBox->AddChild(dataValTC);
	viewRect.OffsetTo(msgEnabledRB->Frame().left, dataValTC->Frame().bottom + 5);
	viewRect.right = msgBox->Bounds().right - msgEnabledRB->Frame().left;
	dataNameTC = new BTextControl(viewRect, "Data Name", "Data Name:", NULL, B_FOLLOW_NONE, B_WILL_DRAW);
	dataNameTC->SetDivider(be_plain_font->StringWidth(dataNameTC->Label()) + 4);
	msgBox->AddChild(dataNameTC);
	viewRect.OffsetTo(msgEnabledRB->Frame().left, dataNameTC->Frame().bottom + 7);
	appSigSV = new BStringView(viewRect, "App Sig", "Signature:", B_FOLLOW_NONE, B_WILL_DRAW);
	msgBox->AddChild(appSigSV);
	viewRect.OffsetTo(msgEnabledRB->Frame().left, appSigSV->Frame().bottom + 5);
	appPathSV = new BStringView(viewRect, "App Path", "Path:", B_FOLLOW_NONE, B_WILL_DRAW);
	msgBox->AddChild(appPathSV);
	msgBox->ResizeTo(msgBox->Bounds().Width(), appPathSV->Frame().bottom + 5);
	mainView->AddChild(msgBox);
	msgBox->Hide();
	//About Box
	aboutBox = new BBox(msgBox->Frame(), "About ReloadDeskbarIcons", B_FOLLOW_NONE, B_WILL_DRAW);
	aboutBox->SetLabel("About ReloadDeskbarIcons");
	viewRect = aboutBox->Bounds();
	viewRect.InsetBy(10, 12);
	BRect textRect(viewRect);
	textRect.OffsetTo(0,3);
	aboutTView = new BTextView(viewRect, "About View", textRect, B_FOLLOW_NONE, B_WILL_DRAW);
	aboutTView->SetViewColor(bg_color);
	aboutTView->SetText(
		"ReloadDeskbarIcons v1.01 by Brian Hill\n\n"
		"Use this application to configure a list of BMessages to reload Deskbar "
		"application icons.  This app is a perfect addition to DeskAttendant "
		"for reloading icons which can't be reloaded via command line options.\n\n"
		"Currently settings for BeAIM and GimICQ are included.  If you have "
		"information on reloading other application icons with BMessages please "
		"send the necessary info to me at Perelandra@iname.com.  I will update "
		"this app with new included settings as I get them.\n\n"
		"The newest version of this application can be found at www.BeBits.com or "
		"my page at http://homepage.mac.com/perelandra777/beossoftware.htm.");
	aboutTView->MakeEditable(false);
	aboutTView->MakeSelectable(false);
	aboutBox->AddChild(aboutTView);
	mainView->AddChild(aboutBox);
	AddChild(mainView);
	ResizeTo(msgUpB->Frame().right + BORDER_SIZE, msgBox->Frame().bottom + BORDER_SIZE);
	//Initial settings
	recallMsgSettings();
	//File Panel
	appFilter = new RDIAppFilter();
	addPanel = new BFilePanel(B_OPEN_PANEL, NULL, NULL, B_FILE_NODE, false, NULL, appFilter);
	//Saved settings
	BPath settingsPath(RDIsettingsDir);
	BDirectory settingsDir;
	if( settingsDir.SetTo(settingsPath.Path()) == B_ENTRY_NOT_FOUND )
	{	//(new BAlert("","Settings directory not found, creating directory "
						//"~/config/settings/RedBear/DeskAttendant", "OK"))->Go();
		if(settingsDir.CreateDirectory(settingsPath.Path(), &settingsDir)!=B_OK)
		{	(new BAlert("","Error creating settings folder","OK"))->Go(); }
	}
	BEntry archiveEntry("/boot/home/config/settings/ReloadDeskbarIcons/startup");
	if(archiveEntry.Exists())
	{	BFile archiveFile(&archiveEntry, B_READ_ONLY);
		off_t datasize; archiveFile.GetSize(&datasize);
		char *databuf = new char[datasize];
		BMessage archive;
		if((archiveFile.Read(databuf, datasize))!=datasize)
		{	//(new BAlert("","Error loading archive file","OK",
			//				NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT))->Go();
		}
		else if(archive.Unflatten(databuf)!=B_OK)
		{	//(new BAlert("","Error reading archived settings","OK",
			//				NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT))->Go();
		}
		else
		{	float windowTop, windowLeft;
			if(archive.FindFloat("DAP:windowTop",&windowTop) == B_OK &&
				archive.FindFloat("DAP:windowLeft",&windowLeft) == B_OK);
				MoveTo(windowLeft,windowTop);
			restoreSettings(archive);
		}
		delete[] databuf;
	}
	else//no startup file
	{	addIncludedMsgs(0);
	}
	Unlock();
	//Show();
}
RDIWindow::~RDIWindow()
{	//Remove List Items
	MsgListItem *msgItem;
	DataListItem *dataItem;
	int32 count;
	do
	{	msgItem = (MsgListItem*)msgLView->RemoveItem(int32(0));
		if(msgItem)
		{	count = msgItem->dataList.CountItems();
			for(int i=0; i<count; i++)
			{	dataItem = (DataListItem*)msgItem->dataList.RemoveItem(int32(0));
				delete dataItem;
			}
			delete msgItem;
		}
	}while(msgItem);
	delete addPanel;
	delete appFilter;
}
bool RDIWindow::QuitRequested()
{	BMessage archive;
	archive.AddFloat("DAP:windowTop", Frame().top);
	archive.AddFloat("DAP:windowLeft", Frame().left);
	saveMsgSettings();//in case selected item hasn't been saved yet
	storeSettings(archive);
	BFile saveFile("/boot/home/config/settings/ReloadDeskbarIcons/startup",
					B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE);
	int datasize = archive.FlattenedSize();
	char *databuf = new char[datasize];
	if(archive.Flatten(databuf, datasize)!=B_OK)
	{	(new BAlert("","Error flattening data archive.","OK",
						NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT))->Go();
	}
	else if( (saveFile.Write(databuf, datasize))!=datasize )
	{	(new BAlert("","Error writing archive file.","OK",
						NULL, NULL, B_WIDTH_AS_USUAL, B_STOP_ALERT))->Go();
	}
	delete[] databuf;
	be_app->PostMessage(B_QUIT_REQUESTED);
	return(true);
}
void RDIWindow::MessageReceived(BMessage* msg)
{	switch(msg->what)
	{	case MOVE_MSG_UP: {
			int32 index = msgLView->CurrentSelection();
			msgLView->MoveItem(index, index-1);
			saveMsgSettings();
			clearMsgSettings();
			updateSelectedMsg();
			recallMsgSettings();
			break; }
		case MOVE_MSG_DOWN: {
			int32 index = msgLView->CurrentSelection();
			msgLView->MoveItem(index, index+1);
			saveMsgSettings();
			clearMsgSettings();
			updateSelectedMsg();
			recallMsgSettings();
			break; }
		case ADD_MSG: {
			BMessage addmsg(ADD_MSG_REF);
			addPanel->SetMessage(&addmsg);
			addPanel->SetTarget(this);
			addPanel->Show();
			break; }
		case ADD_MSG_REF: {
			Lock();
			entry_ref srcRef;
			msg->FindRef("refs", &srcRef);
			BEntry srcEntry(&srcRef, true);
			BPath path(&srcEntry);
			BNode node(&srcEntry);
			char *buf = new char[B_ATTR_NAME_LENGTH];
			ssize_t size;
			if( (size = node.ReadAttr("BEOS:APP_SIG",0,0,buf,B_ATTR_NAME_LENGTH)) > 0 )
			{	MsgListItem* item = new MsgListItem(buf, path, "");
				msgLView->AddItem(item);
				saveMsgSettings();
				clearMsgSettings();
				updateSelectedMsg();
				recallMsgSettings();
			}
			else (new BAlert("","Excecutable does not have an application signature","OK"))->Go();
			delete[] buf;
			Unlock();
			break; }
		case REMOVE_MSG: {
			int32 count = msgLView->CountItems();
			MsgListItem *item;
			for(int i=count-1; i>=0; i--)
			{	if(msgLView->IsItemSelected(i)) item = (MsgListItem*)(msgLView->RemoveItem(i)); }
			saveMsgSettings();
			clearMsgSettings();
			updateSelectedMsg();
			recallMsgSettings();
			delete item;
			break; }
		case ADD_INC_ALL: { addIncludedMsgs(0); break; }
		case ADD_INC_BEAIM: { addIncludedMsgs(1); break; }
		case ADD_INC_GIMICQ: { addIncludedMsgs(2); break; }
		case ADD_INC_ICBM: { addIncludedMsgs(3); break; }
		case SEND_CUR_MSG: {
			saveMsgSettings();
			int32 index = msgLView->CurrentSelection();
			if(index<0) break;
			sendMessage((MsgListItem*)msgLView->ItemAt(index));
			break; }
		case SEND_ALL_MSG: {
			saveMsgSettings();
			sendAllMessages();
			break; }
		case MSG_SELECTION_CHANGED: {
			Lock();
			saveMsgSettings();
			clearMsgSettings();
			updateSelectedMsg();
			recallMsgSettings();
			Unlock();
			break; }
		case TOGGLE_ENABLED: {
			if(selectedMType == MSG_TYPE)
			{	selectedMsg->enabled = msgEnabledRB->Value();
				msgLView->InvalidateItem(msgLView->IndexOf(selectedMsg));
			}
			break; }
		case NEW_DATA_ITEM: {
			dataCLView->AddItem(new DataListItem(0, "Name", "Value"));
			break; }
		case REMOVE_DATA_ITEM: {
			int32 count = dataCLView->CountItems();
			DataListItem *item;
			for(int i=count-1; i>=0; i--)
			{	if(dataCLView->IsItemSelected(i))
					item = (DataListItem*)(dataCLView->RemoveItem(i));
			}
			updateSelectedItem();
			delete item;
			break; }
		case DATA_SELECTION_CHANGED: {
			Lock();
			saveItemSettings();
			clearItemSettings();
			updateSelectedItem();
			recallItemSettings();
			Unlock();
			break; }
		default: { break; }
	}
	return;
}
void RDIWindow::saveMsgSettings()//save user configureable settings
{	if(selectedMsg != NULL)
	{	selectedMsg->enabled = msgEnabledRB->Value();
		selectedMsg->msgCmd.SetTo(msgCmdTC->Text());
		selectedMsg->msgName.SetTo(msgNameTC->Text());
		BPath path(selectedMsg->appPath.String());
		selectedMsg->name.SetTo(path.Leaf());
		if(strcmp(selectedMsg->msgName.String(),"")!=0)
		{	selectedMsg->name.Append(": ");
			selectedMsg->name.Append(selectedMsg->msgName.String());
		}
		saveItemSettings();
		selectedMsg->dataList.MakeEmpty();
		int32 count = dataCLView->CountItems();
		for(int i=0; i<count; i++)
		{	selectedMsg->dataList.AddItem(dataCLView->ItemAt(i)); }
		msgLView->InvalidateItem(msgLView->IndexOf(selectedMsg));
	}
	return;
}
void RDIWindow::clearMsgSettings()
{	if(selectedMType == MSG_TYPE)
	{	msgEnabledRB->SetValue(0);
		msgDisabledRB->SetValue(0);
		msgCmdTC->SetText("");
		msgNameTC->SetText("");
		dataCLView->MakeEmpty();
		appSigSV->SetText("App Signature:");
		appPathSV->SetText("App Path:");
		clearItemSettings();
	}
	return;
}
void RDIWindow::updateSelectedMsg()
{	int32 index = msgLView->CurrentSelection();
	if(index < 0)//No selection
	{	if(selectedMType == MSG_TYPE)
		{	msgBox->Hide();
			aboutBox->Show();
		}
		selectedMsg = NULL;
		selectedMType = 0;
	}
	else//Item selected
	{	MsgListItem *newitem = (MsgListItem*)msgLView->ItemAt(index);
		if(selectedMType == 0)
		{	aboutBox->Hide();
			msgBox->Show();
		}
		selectedMsg = newitem;
		selectedMType = MSG_TYPE;
		updateSelectedItem();
	}
	return;
}
void RDIWindow::recallMsgSettings()
{	//Enable-disable objects
	if(selectedMType == 0)//no selected item
	{	msgUpB->SetEnabled(false);
		msgDownB->SetEnabled(false);
		msgRemB->SetEnabled(false);
	}
	else
	{	//Enable buttons
		int32 index = msgLView->IndexOf(selectedMsg);
		msgUpB->SetEnabled(index > 0);
		msgDownB->SetEnabled( (msgLView->CountItems() - index) > 1 );
		msgRemB->SetEnabled(selectedMsg->removable);
		dataNewB->SetEnabled(selectedMsg->editable);
		//Settings
		if(selectedMsg->enabled) msgEnabledRB->SetValue(1);
		else msgDisabledRB->SetValue(1);
		msgCmdTC->SetText(selectedMsg->msgCmd.String());
		msgCmdTC->SetEnabled(selectedMsg->editable);
		msgNameTC->SetText(selectedMsg->msgName.String());
		msgNameTC->SetEnabled(selectedMsg->editable);
		BString path("App Signature: ");
		path.Append(selectedMsg->appSig.String());
		appSigSV->SetText(path.String());
		path.SetTo("App Path: ");
		path.Append(selectedMsg->appPath.String());
		appPathSV->SetText(path.String());
		//Messages
		dataCLView->AddList(&(selectedMsg->dataList));
		recallItemSettings();
	}
	return;
}
void RDIWindow::saveItemSettings()
{	if(selectedItem != NULL)
	{	selectedItem->dataName.SetTo(dataNameTC->Text());
		selectedItem->dataVal.SetTo(dataValTC->Text());
		selectedItem->dataType = dataTypeMenu->IndexOf(dataTypeMenu->FindMarked());
		selectedItem->updateColumns();
		dataCLView->InvalidateItem(dataCLView->IndexOf(selectedItem));
	}
	return;
}
void RDIWindow::clearItemSettings()
{	if(selectedIType == DATA_TYPE)
	{	dataTypeMenu->ItemAt(0)->SetMarked(true);
		dataValTC->SetText("");
		dataNameTC->SetText("");
	}
	return;
}
void RDIWindow::updateSelectedItem()
{	int32 index = dataCLView->CurrentSelection();
	if(index < 0)//No selection
	{	selectedItem = NULL;
		selectedIType = 0;
	}
	else//Item selected
	{	DataListItem *newitem = (DataListItem*)dataCLView->ItemAt(index);
		selectedItem = newitem;
		selectedIType = DATA_TYPE;
	}
	return;
}
void RDIWindow::recallItemSettings()
{	if(selectedIType == 0)//no selected item
	{	dataRemB->SetEnabled(false);
		dataValTC->SetEnabled(false);
		dataTypeField->SetEnabled(false);
		dataNameTC->SetEnabled(false);
	}
	else
	{	//Settings
		dataTypeMenu->ItemAt(selectedItem->dataType)->SetMarked(true);
		dataNameTC->SetText(selectedItem->dataName.String());
		dataValTC->SetText(selectedItem->dataVal.String());
		//Disable items
		dataRemB->SetEnabled(selectedItem->removable);
		dataValTC->SetEnabled(selectedItem->editable);
		dataTypeField->SetEnabled(selectedItem->editable);
		dataNameTC->SetEnabled(selectedItem->editable);
	}
	return;
}
BEntry RDIWindow::getEntryFromSig(char* sig)
{	BVolumeRoster v_roster;
	BVolume vol;
	BQuery query;
	BEntry entry;
	BString pred("BEOS:APP_SIG==");
	pred.Append(sig);
	v_roster.Rewind();
	while(v_roster.GetNextVolume(&vol)==B_NO_ERROR)
	{	if(vol.KnowsAttr() && vol.KnowsMime() && vol.KnowsQuery())
		{	query.Clear();
			query.SetPredicate(pred.String());
			query.SetVolume(&vol);
			if(query.Fetch()!=B_OK) continue;
			if(query.GetNextEntry(&entry)==B_OK)//entry found
			{	break; }
		}
	}
	return entry;
}
void RDIWindow::addIncludedMsgs(int which)
{	MsgListItem *newmsg;
	//DataListItem *newitem;
	BEntry entry;
	BPath path;
	BString app_sig;
	bool single = true;
	switch(which)
	{	case 0: single = false;
		case 1://BeAIM
		{	app_sig.SetTo("application/x-vnd.FifthAce-BeAIM");
			entry = getEntryFromSig((char*)app_sig.String());
			if(entry.Exists())//entry found
			{	entry.GetPath(&path);
				newmsg = new MsgListItem((char*)app_sig.String(), path, "Remove Icon", 1, 1, 0);
				newmsg->msgCmd.SetTo("SQRL");
				newmsg->dataList.AddItem(new DataListItem(0, "enabled", "false", 0, 0));
				msgLView->AddItem(newmsg);
				newmsg = new MsgListItem((char*)app_sig.String(), path, "Load Icon", 1, 1, 0);
				newmsg->msgCmd.SetTo("SQRL");
				newmsg->dataList.AddItem(new DataListItem(0, "enabled", "true", 0, 0));
				msgLView->AddItem(newmsg);
			}
			if(single) break;
		}
		case 2://GimICQ
		{	app_sig.SetTo("application/x-vnd.Gimmick-GimICQ2");
			entry = getEntryFromSig((char*)app_sig.String());
			if(entry.Exists())//entry found
			{	entry.GetPath(&path);
				newmsg = new MsgListItem((char*)app_sig.String(), path, "Remove Icon", 1, 1, 0);
				newmsg->msgCmd.SetTo("SQRL");
				newmsg->dataList.AddItem(new DataListItem(0, "enabled", "false", 0, 0));
				msgLView->AddItem(newmsg);
				newmsg = new MsgListItem((char*)app_sig.String(), path, "Load Icon", 1, 1, 0);
				newmsg->msgCmd.SetTo("SQRL");
				newmsg->dataList.AddItem(new DataListItem(0, "enabled", "true", 0, 0));
				msgLView->AddItem(newmsg);
			}
			if(single) break;
		}
		case 3://ICBM
		{	app_sig.SetTo("application/x-vnd.ICBM");
			entry = getEntryFromSig((char*)app_sig.String());
			if(entry.Exists())//entry found
			{	entry.GetPath(&path);
				newmsg = new MsgListItem((char*)app_sig.String(), path, "Reload Icon", 1, 1, 0);
				newmsg->msgCmd.SetTo("SQRL");
				newmsg->dataList.AddItem(new DataListItem(0, "enabled", "true", 0, 0));
				msgLView->AddItem(newmsg);
			}
			if(single) break;
		}
	}
	return;
}
void RDIWindow::sendAllMessages()
{	MsgListItem* msgitem;
	int32 count = msgLView->CountItems();
	for(int i=0; i<count; i++)
	{	msgitem = (MsgListItem*)msgLView->ItemAt(i);
		if(msgitem->enabled) sendMessage(msgitem);
	}
	return;
}
void RDIWindow::sendMessage(MsgListItem* msgitem)
{	BRoster app_roster;
	entry_ref app_entry;
	if(app_roster.FindApp(msgitem->appSig.String(), &app_entry) != B_OK) return;
	BMessenger messenger(msgitem->appSig.String());
	//Convert char* to uint32
	uint32 cmd =	toascii(msgitem->msgCmd[0]) * 0x1000000 +
					toascii(msgitem->msgCmd[1]) * 0x10000 +
					toascii(msgitem->msgCmd[2]) * 0x100 +
					toascii(msgitem->msgCmd[3]);
	BMessage send_msg(cmd);
	//Add message items
	int32 count = msgitem->dataList.CountItems();
	DataListItem* dataitem;
	for(int i=0; i<count; i++)
	{	dataitem = (DataListItem*)msgitem->dataList.ItemAt(i);
		switch(dataitem->dataType)
		{	case 0: {//Bool
				bool val;
				if(dataitem->dataVal.IFindFirst("false") != B_ERROR)
				{	val = false; }
				else if(dataitem->dataVal.IFindFirst("true") != B_ERROR)
				{	val = true; }
				else
				{	(new BAlert("","Error adding data to message","OK"))->Go();
					return; }
				send_msg.AddBool(dataitem->dataName.String(), val);
				break; }
			case 1: {//Int8
				int8 val;
				val = atoi(dataitem->dataVal.String());
				send_msg.AddInt8(dataitem->dataName.String(), val);
				break; }
			case 2: {//Int16
				int16 val;
				val = atoi(dataitem->dataVal.String());
				send_msg.AddInt16(dataitem->dataName.String(), val);
				break; }
			case 3: {//Int32
				int32 val;
				val = atol(dataitem->dataVal.String());
				send_msg.AddInt32(dataitem->dataName.String(), val);
				break; }
			case 4: {//Int64
				int64 val;
				strtoll (dataitem->dataVal.String(), (char**)0, 10);
				send_msg.AddInt64(dataitem->dataName.String(), val);
				break; }
			case 5: {//Float
				float val;
				val = atof(dataitem->dataVal.String());
				send_msg.AddFloat(dataitem->dataName.String(), val);
				break; }
			case 6: {//Double
				double val;
				val = atof(dataitem->dataVal.String());
				send_msg.AddDouble(dataitem->dataName.String(), val);
				break; }
			case 7: {//String
				send_msg.AddString(dataitem->dataName.String(), dataitem->dataVal.String());
				break; }
		}
	}
	//Send message
	messenger.SendMessage(&send_msg);
	return;
}
void RDIWindow::storeSettings(BMessage &archive)
{	//Application List
	MsgListItem* msgItem;
	DataListItem* dataItem;
	int32 itemNumber = msgLView->CountItems();
	archive.AddInt32("RDI:appListNumber", itemNumber);
	for(int32 i=0; i<itemNumber; i++)
	{	msgItem = (MsgListItem*)msgLView->ItemAt(i);
		archive.AddBool("RDI:appEnabled", msgItem->enabled);
		archive.AddBool("RDI:appRemovable", msgItem->removable);
		archive.AddBool("RDI:appEditable", msgItem->editable);
		archive.AddString("RDI:appSig", msgItem->appSig.String());
		archive.AddString("RDI:msgCmd", msgItem->msgCmd.String());
		archive.AddString("RDI:msgNam", msgItem->msgName.String());
		int32 count = msgItem->dataList.CountItems();
		archive.AddInt32("RDI:msgNumber", count);
		for(int j=0; j<count; j++)
		{	dataItem = (DataListItem*)msgItem->dataList.ItemAt(j);
			archive.AddBool("RDI:msgRemovable", dataItem->removable);
			archive.AddBool("RDI:msgEditable", dataItem->editable);
			archive.AddInt32("RDI:msgType", dataItem->dataType);
			archive.AddString("RDI:msgName", dataItem->dataName.String());
			archive.AddString("RDI:msgVal", dataItem->dataVal.String());
		}
	}
	return;
}
void RDIWindow::restoreSettings(BMessage &archive)
{	Lock();
	int32 int32_value;
	if(archive.FindInt32("RDI:appListNumber", &int32_value) == B_OK)
	{	char *sig, *cmd, *name, *dataval;
		BPath app_path;
		BEntry app_entry;
		bool en, rem, ed, datarem, dataed;
		int dataIndex = 0;
		int32 dataCount, datatype;
		MsgListItem *item;
		for(int32 i=0; i<int32_value; i++)
		{	archive.FindString("RDI:appSig", i, (const char**)&sig);
			app_entry = getEntryFromSig(sig);
			if(!app_entry.Exists())
			{	BString mess("Application ");
				mess.Append(sig);
				mess.Append(" not found.");
				(new BAlert("",mess.String(), "OK"))->Go();
			}
			else
			{	//Message settings
				archive.FindBool("RDI:appEnabled", i, &en);
				archive.FindBool("RDI:appRemovable", i, &rem);
				archive.FindBool("RDI:appEditable", i, &ed);
				archive.FindString("RDI:msgCmd", i, (const char**)&cmd);
				archive.FindString("RDI:msgNam", i, (const char**)&name);
				app_path.SetTo(&app_entry);
				item = new MsgListItem(sig, app_path, name, en, rem, ed);
				item->msgCmd.SetTo(cmd);
				//Message format
				archive.FindInt32("RDI:msgNumber", i, &dataCount);
				for(int i=0; i<dataCount; i++)
				{	archive.FindBool("RDI:msgRemovable", dataIndex, &datarem);
					archive.FindBool("RDI:msgEditable", dataIndex, &dataed);
					archive.FindInt32("RDI:msgType", dataIndex, &datatype);
					archive.FindString("RDI:msgName", dataIndex, (const char**)&name);
					archive.FindString("RDI:msgVal", dataIndex, (const char**)&dataval);
					item->dataList.AddItem(new DataListItem(int(datatype), name, dataval, datarem, dataed));
					dataIndex++;
				}
				
				msgLView->AddItem(item);
			}
		}
	}
	Unlock();
	return;
}
MsgListItem::MsgListItem(char* sig, BPath& path, char* nam, bool en=true, bool rem=true, bool edit=true)
	:BListItem(), enabled(en), removable(rem), editable(edit)
	,name(""), appPath(""), appSig(""), msgCmd(""), msgName(""), dataList()
{	appSig.SetTo(sig);
	appPath.SetTo(path.Path());
	msgName.SetTo(nam);
	name.SetTo(path.Leaf());
	if(strcmp(msgName.String(),"")!=0)
	{	name.Append(": ");
		name.Append(msgName.String());
	}
}
MsgListItem::~MsgListItem()
{	}
void MsgListItem::DrawItem(BView* owner, BRect item_rect, bool complete)
{	rgb_color color;
	bool selected = IsSelected();
	if(selected) color = selected_color;
	else color = owner->ViewColor();
	owner->SetLowColor(color);
	owner->SetDrawingMode(B_OP_COPY);
	if(selected || complete)
	{	owner->SetHighColor(color);
		owner->FillRect(item_rect);
	}
	//BRegion Region;
	//Region.Include(item_column_rect);
	//owner->ConstrainClippingRegion(&Region);
	if(enabled) owner->SetHighColor(enabled_color);
	else owner->SetHighColor(disabled_color);
	owner->DrawString(name.String(),
			BPoint(item_rect.left+5.0,item_rect.bottom - 2.0));
	//owner->ConstrainClippingRegion(NULL);
}
DataListItem::DataListItem(int ty, char* name, char* val, bool rem=true, bool edit=true)
	:CLVEasyItem(), removable(rem), editable(edit)
	,dataType(ty), dataName(name), dataVal(val)
{	updateColumns();
}
DataListItem::~DataListItem()
{	}
void DataListItem::updateColumns()
{	BString type_s;
	switch(dataType)
	{	case 0: type_s.SetTo("Bool"); break;
		case 1: type_s.SetTo("Int8"); break;
		case 2: type_s.SetTo("Int16"); break;
		case 3: type_s.SetTo("Int32"); break;
		case 4: type_s.SetTo("Int64"); break;
		case 5: type_s.SetTo("Float"); break;
		case 6: type_s.SetTo("Double"); break;
		case 7: type_s.SetTo("String"); break;
	}
	SetColumnContent(0,type_s.String());
	SetColumnContent(1,dataName.String());
	SetColumnContent(2,dataVal.String());
	return;
}
bool RDIAppFilter::Filter(const entry_ref *ref, BNode *node, struct stat *st,
							const char *filetype)
{	if(strcmp("application/x-vnd.Be-directory", filetype)==0) { return true; }//folders
	else if(strcmp("application/x-vnd.Be-volume", filetype)==0) { return true; }//volumes
	else if(strcmp("application/x-vnd.Be-elfexecutable", filetype)==0) { return true; }//apps
	else if(strcmp("application/x-vnd.Be-symlink", filetype)==0)//symlinks
	{	BEntry linkedEntry(ref, true);
		if(linkedEntry.InitCheck()!=B_OK) { return false; }
		BNode linkedNode(&linkedEntry);
		if(linkedNode.InitCheck()!=B_OK) { return false; }
		BNodeInfo linkedNodeInfo(&linkedNode);
		if(linkedNodeInfo.InitCheck()!=B_OK) { return false; }
		char *type = new char[B_ATTR_NAME_LENGTH];
		bool pass(false);
		if(linkedNodeInfo.GetType(type)!=B_OK) { pass = false; }
		else if(strcmp("application/x-vnd.Be-directory", type)==0) { pass = true; }//folders
		else if(strcmp("application/x-vnd.Be-volume", type)==0) { pass = true; }//volumes
		else if(strcmp("application/x-vnd.Be-elfexecutable", type)==0) { pass = true; }//apps
		delete[] type;
		return pass;
	}
	return false;
}