/*  Couch Potato's Planner- written by Brian Hill, perelandra@iname.com
    
    This program was written for BeOS (www.be.com) and provides an interface
    for searching the internet for TV schedules of your favorite programs.
    
    
    --Protected under General Public License--
    
    Copyright (C) 2001,  Brian Hill <perelandra@iname.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef RELOADDESKBARICONS_H
#define RELOADDESKBARICONS_H
#include <AppKit.h>
#include <InterfaceKit.h>
#include <SupportKit.h>
#include <StorageKit.h>
#include <stdlib.h>
#include <ctype.h>
#include <BetterScrollView.h>
#include <ColumnListView.h>
#include <CLVEasyItem.h>
#define BORDER_SIZE 5
#define MSG_TYPE 'msgt'
#define DATA_TYPE 'datt'

class RDIWindow;
class MsgListItem;
class DataListItem;
class RDIAppFilter;
class RDIApp : public BApplication
{public:
					RDIApp(bool);
					~RDIApp();
	virtual void	ReadyToRun();
private:
	RDIWindow		*aWindow;
	bool			reload;
};
class RDIWindow : public BWindow
{public:
					RDIWindow(BRect);
					~RDIWindow();
	virtual	bool	QuitRequested();
	virtual void	MessageReceived(BMessage*);
private:
	RDIAppFilter	*appFilter;
	BFilePanel		*addPanel;
	MsgListItem		*selectedMsg;
	DataListItem	*selectedItem;
	int32			selectedMType, selectedIType;
	BView			*mainView;
	BListView		*msgLView;
	BetterScrollView *msgSView;
	BPopUpMenu		*includedItemsMenu, *dataTypeMenu;
	BMenuField		*itemField, *dataTypeField;
	CLVContainerView *dataCView;
	ColumnListView	*dataCLView;
	BBox			*msgBox, *aboutBox;
	BButton			*msgUpB, *msgDownB, *msgAddB, *msgRemB, *dataNewB, *dataRemB,
					*msgSendB;
	BRadioButton	*msgEnabledRB, *msgDisabledRB;
	BTextControl	*msgCmdTC, *msgNameTC, *dataNameTC, *dataValTC;
	BStringView		*appSigSV, *appPathSV;
	BTextView		*aboutTView;
	void			saveMsgSettings();
	void			clearMsgSettings();
	void			updateSelectedMsg();
	void			recallMsgSettings();
	void			saveItemSettings();
	void			clearItemSettings();
	void			updateSelectedItem();
	void			recallItemSettings();
	BEntry			getEntryFromSig(char*);
	void			addIncludedMsgs(int);
	void			sendAllMessages();
	void			sendMessage(MsgListItem*);
	void			storeSettings(BMessage&);
	void			restoreSettings(BMessage&);	
};
class MsgListItem : public BListItem
{public:
					MsgListItem(char*, BPath&, char*, bool=true, bool=true, bool=true);
					~MsgListItem();
private:
	bool			enabled, removable, editable;
	BString			name, appPath, appSig, msgCmd, msgName;
	void			DrawItem(BView*, BRect, bool);
	BList			dataList;
//	void			Update(BView *owner, const BFont *font);
//	virtual bool	InitiateDrag(BPoint point, int32 index, bool wasSelected);
	friend class RDIWindow;
};
class DataListItem : public CLVEasyItem
{public:
					DataListItem(int, char*, char*, bool=true, bool=true);
					~DataListItem();
	void			updateColumns();
private:
	bool			removable, editable;
	int				dataType;
	BString			dataName, dataVal;
	friend class RDIWindow;
};
class RDIAppFilter : public BRefFilter
{public:
					//DAPappFilter();
					//~DAPappFilter();
	virtual bool	Filter(const entry_ref *ref, BNode *node, struct stat *st,
							const char *filetype);
};
#define MSG_SELECTION_CHANGED 'mchg'
#define MOVE_MSG_UP		'movu'
#define MOVE_MSG_DOWN	'movd'
#define ADD_MSG			'addm'
#define ADD_MSG_REF		'addr'
#define REMOVE_MSG		'remm'
#define SEND_CUR_MSG	'scms'
#define SEND_ALL_MSG	'sams'
#define TOGGLE_ENABLED	'toge'
#define DATA_SELECTION_CHANGED 'dchg'
#define NEW_DATA_ITEM	'ndat'
#define REMOVE_DATA_ITEM 'remd'
#define ADD_INC_ALL		'aial'
#define ADD_INC_BEAIM	'aiba'
#define ADD_INC_GIMICQ	'aigi'
#define ADD_INC_ICBM	'aiic'

#endif